@@ -37,8 +37,6 @@ import javax.naming.NamingException;
 import javax.naming.Reference;
 import javax.naming.StringRefAddr;
 
-import org.apache.catalina.ContainerEvent;
-import org.apache.catalina.ContainerListener;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
@@ -78,8 +76,10 @@ import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Helper class used to initialize and populate the JNDI context associated with each context and server.
+ *
+ * @author Remy Maucherat
  */
-public class NamingContextListener implements LifecycleListener, ContainerListener, PropertyChangeListener {
+public class NamingContextListener implements LifecycleListener, PropertyChangeListener {
 
     private static final Log log = LogFactory.getLog(NamingContextListener.class);
     protected static final StringManager sm = StringManager.getManager(NamingContextListener.class);
@@ -235,7 +235,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                 try {
                     createNamingContext();
                 } catch (NamingException e) {
-                    log.error(sm.getString("naming.namingContextCreationFailed", container), e);
+                    log.error(sm.getString("naming.namingContextCreationFailed", e));
                 }
 
                 namingResources.addPropertyChangeListener(this);
@@ -248,7 +248,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                         ContextBindings.bindClassLoader(container, token,
                                 ((Context) container).getLoader().getClassLoader());
                     } catch (NamingException e) {
-                        log.error(sm.getString("naming.bindFailed", container), e);
+                        log.error(sm.getString("naming.bindFailed", e));
                     }
                 }
 
@@ -257,7 +257,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                     try {
                         ContextBindings.bindClassLoader(container, token, this.getClass().getClassLoader());
                     } catch (NamingException e) {
-                        log.error(sm.getString("naming.bindFailed", container), e);
+                        log.error(sm.getString("naming.bindFailed", e));
                     }
                     if (container instanceof StandardServer) {
                         ((StandardServer) container).setGlobalNamingContext(namingContext);
@@ -321,23 +321,6 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
     }
 
 
-    // ---------------------------------------------- ContainerListener Methods
-
-    /**
-     * NO-OP.
-     *
-     * @param event ContainerEvent that has occurred
-     *
-     * @deprecated The {@link ContainerListener} interface and implementing methods will be removed from this class for
-     *                 Tomcat 10 onwards.
-     */
-    @Deprecated
-    @Override
-    public void containerEvent(ContainerEvent event) {
-        // NO-OP
-    }
-
-
     // ----------------------------------------- PropertyChangeListener Methods
 
 
@@ -589,7 +572,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                 // Ignore because UserTransaction was obviously
                 // added via ResourceLink
             } catch (NamingException e) {
-                log.error(sm.getString("naming.bindFailed", "UserTransaction"), e);
+                log.error(sm.getString("naming.bindFailed", e));
             }
         }
 
@@ -598,7 +581,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
             try {
                 compCtx.bind("Resources", ((Context) container).getResources());
             } catch (NamingException e) {
-                log.error(sm.getString("naming.bindFailed", "Resources"), e);
+                log.error(sm.getString("naming.bindFailed", e));
             }
         }
 
@@ -672,7 +655,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
             createSubcontexts(envCtx, ejb.getName());
             envCtx.bind(ejb.getName(), ref);
         } catch (NamingException e) {
-            log.error(sm.getString("naming.bindFailed", ejb.getName()), e);
+            log.error(sm.getString("naming.bindFailed", e));
         }
     }
 
@@ -772,7 +755,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                 createSubcontexts(envCtx, env.getName());
                 envCtx.bind(env.getName(), value);
             } catch (NamingException e) {
-                log.error(sm.getString("naming.invalidEnvEntryValue", env.getName()), e);
+                log.error(sm.getString("naming.invalidEnvEntryValue", e));
             }
         }
     }
@@ -863,7 +846,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                             log.debug(sm.getString("naming.addSlash", service.getWsdlfile()));
                         }
                     } catch (MalformedURLException e) {
-                        log.error(sm.getString("naming.wsdlFailed", service.getWsdlfile()), e);
+                        log.error(sm.getString("naming.wsdlFailed", e));
                     }
                 }
                 if (wsdlURL == null) {
@@ -898,7 +881,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                             log.debug(sm.getString("naming.addSlash", service.getJaxrpcmappingfile()));
                         }
                     } catch (MalformedURLException e) {
-                        log.error(sm.getString("naming.wsdlFailed", service.getJaxrpcmappingfile()), e);
+                        log.error(sm.getString("naming.wsdlFailed", e));
                     }
                 }
                 if (jaxrpcURL == null) {
@@ -959,7 +942,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
             createSubcontexts(envCtx, service.getName());
             envCtx.bind(service.getName(), ref);
         } catch (NamingException e) {
-            log.error(sm.getString("naming.bindFailed", service.getName()), e);
+            log.error(sm.getString("naming.bindFailed", e));
         }
     }
 
@@ -994,7 +977,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
             createSubcontexts(envCtx, resource.getName());
             envCtx.bind(resource.getName(), ref);
         } catch (NamingException e) {
-            log.error(sm.getString("naming.bindFailed", resource.getName()), e);
+            log.error(sm.getString("naming.bindFailed", e));
         }
 
         if (("javax.sql.DataSource".equals(ref.getClassName()) ||
@@ -1006,7 +989,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
                 Registry.getRegistry(null).registerComponent(actualResource, on, null);
                 objectNames.put(resource.getName(), on);
             } catch (Exception e) {
-                log.warn(sm.getString("naming.jmxRegistrationFailed", resource.getName()), e);
+                log.warn(sm.getString("naming.jmxRegistrationFailed", e));
             }
             // Bug 63210. DBCP2 DataSources require an explicit close. This goes
             // further and cleans up and AutoCloseable DataSource by default.
@@ -1046,7 +1029,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
             createSubcontexts(envCtx, resourceEnvRef.getName());
             envCtx.bind(resourceEnvRef.getName(), ref);
         } catch (NamingException e) {
-            log.error(sm.getString("naming.bindFailed", resourceEnvRef.getName()), e);
+            log.error(sm.getString("naming.bindFailed", e));
         }
     }
 
@@ -1078,7 +1061,7 @@ public class NamingContextListener implements LifecycleListener, ContainerListen
             createSubcontexts(envCtx, resourceLink.getName());
             ctx.bind(resourceLink.getName(), ref);
         } catch (NamingException e) {
-            log.error(sm.getString("naming.bindFailed", resourceLink.getName()), e);
+            log.error(sm.getString("naming.bindFailed", e));
         }
 
         ResourceLinkFactory.registerGlobalResourceAccess(getGlobalNamingContext(), resourceLink.getName(),
