@@ -60,8 +60,7 @@ import org.apache.tomcat.util.res.StringManager;
 
 
 /**
- * Holds and manages the naming resources defined in the J2EE Enterprise Naming Context and their associated JNDI
- * context.
+ * Holds and manages the naming resources defined in the Jakarta EE Naming Context and their associated JNDI context.
  *
  * @author Remy Maucherat
  */
@@ -514,8 +513,8 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
 
     /**
-     * @return the array of defined environment entries for this web application. If none have been defined, a
-     *             zero-length array is returned.
+     * @return the array of defined environment entries for this web application. If none have been defined, a zero-length
+     *             array is returned.
      */
     public ContextEnvironment[] findEnvironments() {
 
@@ -650,8 +649,8 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
 
     /**
-     * @return the array of resource environment reference names for this web application. If none have been specified,
-     *             a zero-length array is returned.
+     * @return the array of resource environment reference names for this web application. If none have been specified, a
+     *             zero-length array is returned.
      */
     public ContextResourceEnvRef[] findResourceEnvRefs() {
 
@@ -978,10 +977,10 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
         try {
             m = resource.getClass().getMethod(closeMethod, (Class<?>[]) null);
         } catch (SecurityException e) {
-            log.debug(sm.getString("namingResources.cleanupCloseSecurity", closeMethod, name, container), e);
+            log.debug(sm.getString("namingResources.cleanupCloseSecurity", closeMethod, name, container));
             return;
         } catch (NoSuchMethodException e) {
-            log.debug(sm.getString("namingResources.cleanupNoClose", name, container, closeMethod), e);
+            log.debug(sm.getString("namingResources.cleanupNoClose", name, container, closeMethod));
             return;
         }
         try {
