@@ -23,6 +23,8 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.util.ArrayList;
 import java.util.Date;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicLong;
 
 import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
@@ -76,6 +78,7 @@ public class DeltaManager extends ClusterManagerBase {
     private int stateTransferTimeout = 60;
     private boolean sendAllSessions = true;
     private int sendAllSessionsSize = 1000;
+    private boolean enableStatistics = true;
 
     /**
      * wait time between send session block (default 2 sec)
@@ -88,25 +91,25 @@ public class DeltaManager extends ClusterManagerBase {
 
     // -------------------------------------------------------- stats attributes
 
-    private volatile long sessionReplaceCounter = 0;
-    private volatile long counterReceive_EVT_GET_ALL_SESSIONS = 0;
-    private volatile long counterReceive_EVT_ALL_SESSION_DATA = 0;
-    private volatile long counterReceive_EVT_SESSION_CREATED = 0;
-    private volatile long counterReceive_EVT_SESSION_EXPIRED = 0;
-    private volatile long counterReceive_EVT_SESSION_ACCESSED = 0;
-    private volatile long counterReceive_EVT_SESSION_DELTA = 0;
-    private volatile int counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE = 0;
-    private volatile long counterReceive_EVT_CHANGE_SESSION_ID = 0;
-    private volatile long counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER = 0;
-    private volatile long counterSend_EVT_GET_ALL_SESSIONS = 0;
-    private volatile long counterSend_EVT_ALL_SESSION_DATA = 0;
-    private volatile long counterSend_EVT_SESSION_CREATED = 0;
-    private volatile long counterSend_EVT_SESSION_DELTA = 0;
-    private volatile long counterSend_EVT_SESSION_ACCESSED = 0;
-    private volatile long counterSend_EVT_SESSION_EXPIRED = 0;
-    private volatile int counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE = 0;
-    private volatile long counterSend_EVT_CHANGE_SESSION_ID = 0;
-    private volatile int counterNoStateTransferred = 0;
+    private final AtomicLong sessionReplaceCounter = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_GET_ALL_SESSIONS = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_ALL_SESSION_DATA = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_SESSION_CREATED = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_SESSION_EXPIRED = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_SESSION_ACCESSED = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_SESSION_DELTA = new AtomicLong(0);
+    private final AtomicInteger counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE = new AtomicInteger(0);
+    private final AtomicLong counterReceive_EVT_CHANGE_SESSION_ID = new AtomicLong(0);
+    private final AtomicLong counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER = new AtomicLong(0);
+    private final AtomicLong counterSend_EVT_GET_ALL_SESSIONS = new AtomicLong(0);
+    private final AtomicLong counterSend_EVT_ALL_SESSION_DATA = new AtomicLong(0);
+    private final AtomicLong counterSend_EVT_SESSION_CREATED = new AtomicLong(0);
+    private final AtomicLong counterSend_EVT_SESSION_DELTA = new AtomicLong(0);
+    private final AtomicLong counterSend_EVT_SESSION_ACCESSED = new AtomicLong(0);
+    private final AtomicLong counterSend_EVT_SESSION_EXPIRED = new AtomicLong(0);
+    private final AtomicInteger counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE = new AtomicInteger(0);
+    private final AtomicLong counterSend_EVT_CHANGE_SESSION_ID = new AtomicLong(0);
+    private final AtomicInteger counterNoStateTransferred = new AtomicInteger(0);
 
 
     // ------------------------------------------------------------- Constructor
@@ -130,98 +133,98 @@ public class DeltaManager extends ClusterManagerBase {
      * @return Returns the counterSend_EVT_GET_ALL_SESSIONS.
      */
     public long getCounterSend_EVT_GET_ALL_SESSIONS() {
-        return counterSend_EVT_GET_ALL_SESSIONS;
+        return counterSend_EVT_GET_ALL_SESSIONS.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_SESSION_ACCESSED.
      */
     public long getCounterSend_EVT_SESSION_ACCESSED() {
-        return counterSend_EVT_SESSION_ACCESSED;
+        return counterSend_EVT_SESSION_ACCESSED.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_SESSION_CREATED.
      */
     public long getCounterSend_EVT_SESSION_CREATED() {
-        return counterSend_EVT_SESSION_CREATED;
+        return counterSend_EVT_SESSION_CREATED.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_SESSION_DELTA.
      */
     public long getCounterSend_EVT_SESSION_DELTA() {
-        return counterSend_EVT_SESSION_DELTA;
+        return counterSend_EVT_SESSION_DELTA.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_SESSION_EXPIRED.
      */
     public long getCounterSend_EVT_SESSION_EXPIRED() {
-        return counterSend_EVT_SESSION_EXPIRED;
+        return counterSend_EVT_SESSION_EXPIRED.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_ALL_SESSION_DATA.
      */
     public long getCounterSend_EVT_ALL_SESSION_DATA() {
-        return counterSend_EVT_ALL_SESSION_DATA;
+        return counterSend_EVT_ALL_SESSION_DATA.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE.
      */
     public int getCounterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE() {
-        return counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE;
+        return counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE.get();
     }
 
     /**
      * @return Returns the counterSend_EVT_CHANGE_SESSION_ID.
      */
     public long getCounterSend_EVT_CHANGE_SESSION_ID() {
-        return counterSend_EVT_CHANGE_SESSION_ID;
+        return counterSend_EVT_CHANGE_SESSION_ID.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_ALL_SESSION_DATA.
      */
     public long getCounterReceive_EVT_ALL_SESSION_DATA() {
-        return counterReceive_EVT_ALL_SESSION_DATA;
+        return counterReceive_EVT_ALL_SESSION_DATA.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_GET_ALL_SESSIONS.
      */
     public long getCounterReceive_EVT_GET_ALL_SESSIONS() {
-        return counterReceive_EVT_GET_ALL_SESSIONS;
+        return counterReceive_EVT_GET_ALL_SESSIONS.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_SESSION_ACCESSED.
      */
     public long getCounterReceive_EVT_SESSION_ACCESSED() {
-        return counterReceive_EVT_SESSION_ACCESSED;
+        return counterReceive_EVT_SESSION_ACCESSED.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_SESSION_CREATED.
      */
     public long getCounterReceive_EVT_SESSION_CREATED() {
-        return counterReceive_EVT_SESSION_CREATED;
+        return counterReceive_EVT_SESSION_CREATED.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_SESSION_DELTA.
      */
     public long getCounterReceive_EVT_SESSION_DELTA() {
-        return counterReceive_EVT_SESSION_DELTA;
+        return counterReceive_EVT_SESSION_DELTA.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_SESSION_EXPIRED.
      */
     public long getCounterReceive_EVT_SESSION_EXPIRED() {
-        return counterReceive_EVT_SESSION_EXPIRED;
+        return counterReceive_EVT_SESSION_EXPIRED.get();
     }
 
 
@@ -229,35 +232,35 @@ public class DeltaManager extends ClusterManagerBase {
      * @return Returns the counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE.
      */
     public int getCounterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE() {
-        return counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE;
+        return counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_CHANGE_SESSION_ID.
      */
     public long getCounterReceive_EVT_CHANGE_SESSION_ID() {
-        return counterReceive_EVT_CHANGE_SESSION_ID;
+        return counterReceive_EVT_CHANGE_SESSION_ID.get();
     }
 
     /**
      * @return Returns the counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER.
      */
     public long getCounterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER() {
-        return counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER;
+        return counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER.get();
     }
 
     /**
      * @return Returns the sessionReplaceCounter.
      */
     public long getSessionReplaceCounter() {
-        return sessionReplaceCounter;
+        return sessionReplaceCounter.get();
     }
 
     /**
      * @return Returns the counterNoStateTransferred.
      */
     public int getCounterNoStateTransferred() {
-        return counterNoStateTransferred;
+        return counterNoStateTransferred.get();
     }
 
     public int getReceivedQueueSize() {
@@ -391,6 +394,19 @@ public class DeltaManager extends ClusterManagerBase {
         this.notifyContainerListenersOnReplication = notifyContainerListenersOnReplication;
     }
 
+    /**
+     * @return the enableStatistics
+     */
+    public boolean getEnableStatistics() {
+        return this.enableStatistics;
+    }
+
+    /**
+     * @param enableStatistics the enableStatistics to set
+     */
+    public void setEnableStatistics(boolean enableStatistics) {
+        this.enableStatistics = enableStatistics;
+    }
 
     // --------------------------------------------------------- Public Methods
 
@@ -433,7 +449,9 @@ public class DeltaManager extends ClusterManagerBase {
                 log.trace(sm.getString("deltaManager.sendMessage.newSession", name, sessionId));
             }
             msg.setTimestamp(session.getCreationTime());
-            counterSend_EVT_SESSION_CREATED++;
+            if (enableStatistics) {
+                counterSend_EVT_SESSION_CREATED.incrementAndGet();
+            }
             send(msg);
         }
     }
@@ -495,7 +513,9 @@ public class DeltaManager extends ClusterManagerBase {
                 SessionMessage msg = new SessionMessageImpl(getName(), SessionMessage.EVT_CHANGE_SESSION_ID, data,
                         orgSessionID, orgSessionID + "-" + System.currentTimeMillis());
                 msg.setTimestamp(System.currentTimeMillis());
-                counterSend_EVT_CHANGE_SESSION_ID++;
+                if (enableStatistics) {
+                    counterSend_EVT_CHANGE_SESSION_ID.incrementAndGet();
+                }
                 send(msg);
             } catch (IOException e) {
                 log.error(sm.getString("deltaManager.unableSerializeSessionID", newSessionID), e);
@@ -573,7 +593,9 @@ public class DeltaManager extends ClusterManagerBase {
                 session.resetDeltaRequest();
                 // FIXME How inform other session id cache like SingleSignOn
                 if (findSession(session.getIdInternal()) != null) {
-                    sessionReplaceCounter++;
+                    if (enableStatistics) {
+                        sessionReplaceCounter.incrementAndGet();
+                    }
                     // FIXME better is to grap this sessions again !
                     if (log.isWarnEnabled()) {
                         log.warn(sm.getString("deltaManager.loading.existing.session", session.getIdInternal()));
@@ -684,7 +706,9 @@ public class DeltaManager extends ClusterManagerBase {
             // set reference time
             stateTransferCreateSendTime = beforeSendTime;
             // request session state
-            counterSend_EVT_GET_ALL_SESSIONS++;
+            if (enableStatistics) {
+                counterSend_EVT_GET_ALL_SESSIONS.incrementAndGet();
+            }
             stateTransferred = false;
             // FIXME This send call block the deploy thread, when sender waitForAck is enabled
             try {
@@ -736,7 +760,7 @@ public class DeltaManager extends ClusterManagerBase {
      */
     protected Member findSessionMasterMember() {
         Member mbr = null;
-        Member mbrs[] = cluster.getMembers();
+        Member[] mbrs = cluster.getMembers();
         if (mbrs.length != 0) {
             mbr = mbrs[0];
         }
@@ -777,13 +801,16 @@ public class DeltaManager extends ClusterManagerBase {
                     try {
                         Thread.sleep(100);
                     } catch (Exception sleep) {
+                        // Ignore
                     }
                 } while ((!getStateTransferred()) && (!isNoContextManagerReceived()));
                 reqNow = System.currentTimeMillis();
             }
         }
         if (isTimeout) {
-            counterNoStateTransferred++;
+            if (enableStatistics) {
+                counterNoStateTransferred.incrementAndGet();
+            }
             log.error(sm.getString("deltaManager.noSessionState", getName(), new Date(beforeSendTime),
                     Long.valueOf(reqNow - beforeSendTime)));
         } else if (isNoContextManagerReceived()) {
@@ -819,7 +846,7 @@ public class DeltaManager extends ClusterManagerBase {
         if (log.isInfoEnabled()) {
             log.info(sm.getString("deltaManager.expireSessions", getName()));
         }
-        Session sessions[] = findSessions();
+        Session[] sessions = findSessions();
         for (Session value : sessions) {
             DeltaSession session = (DeltaSession) value;
             if (!session.isValid()) {
@@ -839,12 +866,6 @@ public class DeltaManager extends ClusterManagerBase {
     // -------------------------------------------------------- Replication
     // Methods
 
-    /**
-     * A message was received from another node, this is the callback method to implement if you are interested in
-     * receiving replication messages.
-     *
-     * @param cmsg - the message received.
-     */
     @Override
     public void messageDataReceived(ClusterMessage cmsg) {
         if (cmsg instanceof SessionMessage) {
@@ -872,15 +893,6 @@ public class DeltaManager extends ClusterManagerBase {
         }
     }
 
-    /**
-     * When the request has been completed, the replication valve will notify the manager, and the manager will decide
-     * whether any replication is needed or not. If there is a need for replication, the manager will create a session
-     * message and that will be replicated. The cluster determines where it gets sent.
-     *
-     * @param sessionId - the sessionId that just completed.
-     *
-     * @return a SessionMessage to be sent,
-     */
     @Override
     public ClusterMessage requestCompleted(String sessionId) {
         return requestCompleted(sessionId, false);
@@ -898,7 +910,7 @@ public class DeltaManager extends ClusterManagerBase {
      * @return a SessionMessage to be sent,
      */
     public ClusterMessage requestCompleted(String sessionId, boolean expires) {
-        DeltaSession session = null;
+        DeltaSession session;
         SessionMessage msg = null;
         try {
             session = (DeltaSession) findSession(sessionId);
@@ -908,7 +920,9 @@ public class DeltaManager extends ClusterManagerBase {
                 return null;
             }
             if (session.isDirty()) {
-                counterSend_EVT_SESSION_DELTA++;
+                if (enableStatistics) {
+                    counterSend_EVT_SESSION_DELTA.incrementAndGet();
+                }
                 msg = new SessionMessageImpl(getName(), SessionMessage.EVT_SESSION_DELTA, session.getDiff(), sessionId,
                         sessionId + "-" + System.currentTimeMillis());
             }
@@ -918,7 +932,9 @@ public class DeltaManager extends ClusterManagerBase {
         }
         if (msg == null) {
             if (!expires && !session.isPrimarySession()) {
-                counterSend_EVT_SESSION_ACCESSED++;
+                if (enableStatistics) {
+                    counterSend_EVT_SESSION_ACCESSED.incrementAndGet();
+                }
                 msg = new SessionMessageImpl(getName(), SessionMessage.EVT_SESSION_ACCESSED, null, sessionId,
                         sessionId + "-" + System.currentTimeMillis());
                 if (log.isDebugEnabled()) {
@@ -937,7 +953,9 @@ public class DeltaManager extends ClusterManagerBase {
         if (!expires && (msg == null)) {
             long replDelta = System.currentTimeMillis() - session.getLastTimeReplicated();
             if (session.getMaxInactiveInterval() >= 0 && replDelta > (session.getMaxInactiveInterval() * 1000L)) {
-                counterSend_EVT_SESSION_ACCESSED++;
+                if (enableStatistics) {
+                    counterSend_EVT_SESSION_ACCESSED.incrementAndGet();
+                }
                 msg = new SessionMessageImpl(getName(), SessionMessage.EVT_SESSION_ACCESSED, null, sessionId,
                         sessionId + "-" + System.currentTimeMillis());
                 if (log.isDebugEnabled()) {
@@ -973,25 +991,25 @@ public class DeltaManager extends ClusterManagerBase {
             }
         }
         rejectedSessions = 0;
-        sessionReplaceCounter = 0;
-        counterNoStateTransferred = 0;
+        sessionReplaceCounter.set(0);
+        counterNoStateTransferred.set(0);
         setMaxActive(getActiveSessions());
-        counterReceive_EVT_ALL_SESSION_DATA = 0;
-        counterReceive_EVT_GET_ALL_SESSIONS = 0;
-        counterReceive_EVT_SESSION_ACCESSED = 0;
-        counterReceive_EVT_SESSION_CREATED = 0;
-        counterReceive_EVT_SESSION_DELTA = 0;
-        counterReceive_EVT_SESSION_EXPIRED = 0;
-        counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE = 0;
-        counterReceive_EVT_CHANGE_SESSION_ID = 0;
-        counterSend_EVT_ALL_SESSION_DATA = 0;
-        counterSend_EVT_GET_ALL_SESSIONS = 0;
-        counterSend_EVT_SESSION_ACCESSED = 0;
-        counterSend_EVT_SESSION_CREATED = 0;
-        counterSend_EVT_SESSION_DELTA = 0;
-        counterSend_EVT_SESSION_EXPIRED = 0;
-        counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE = 0;
-        counterSend_EVT_CHANGE_SESSION_ID = 0;
+        counterReceive_EVT_ALL_SESSION_DATA.set(0);
+        counterReceive_EVT_GET_ALL_SESSIONS.set(0);
+        counterReceive_EVT_SESSION_ACCESSED.set(0);
+        counterReceive_EVT_SESSION_CREATED.set(0);
+        counterReceive_EVT_SESSION_DELTA.set(0);
+        counterReceive_EVT_SESSION_EXPIRED.set(0);
+        counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE.set(0);
+        counterReceive_EVT_CHANGE_SESSION_ID.set(0);
+        counterSend_EVT_ALL_SESSION_DATA.set(0);
+        counterSend_EVT_GET_ALL_SESSIONS.set(0);
+        counterSend_EVT_SESSION_ACCESSED.set(0);
+        counterSend_EVT_SESSION_CREATED.set(0);
+        counterSend_EVT_SESSION_DELTA.set(0);
+        counterSend_EVT_SESSION_EXPIRED.set(0);
+        counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE.set(0);
+        counterSend_EVT_CHANGE_SESSION_ID.set(0);
 
     }
 
@@ -1004,7 +1022,9 @@ public class DeltaManager extends ClusterManagerBase {
      */
     protected void sessionExpired(String id) {
         if (cluster.getMembers().length > 0) {
-            counterSend_EVT_SESSION_EXPIRED++;
+            if (enableStatistics) {
+                counterSend_EVT_SESSION_EXPIRED.incrementAndGet();
+            }
             SessionMessage msg = new SessionMessageImpl(getName(), SessionMessage.EVT_SESSION_EXPIRED, null, id,
                     id + "-EXPIRED-MSG");
             msg.setTimestamp(System.currentTimeMillis());
@@ -1019,7 +1039,7 @@ public class DeltaManager extends ClusterManagerBase {
      * Expire all find sessions.
      */
     public void expireAllLocalSessions() {
-        Session sessions[] = findSessions();
+        Session[] sessions = findSessions();
         int expireDirect = 0;
         int expireIndirect = 0;
 
@@ -1124,7 +1144,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @param sender Member which sent the message
      */
     protected void handleALL_SESSION_TRANSFERCOMPLETE(SessionMessage msg, Member sender) {
-        counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE++;
+        if (enableStatistics) {
+            counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE.incrementAndGet();
+        }
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.receiveMessage.transfercomplete", getName(), sender.getHost(),
                     Integer.valueOf(sender.getPort())));
@@ -1143,7 +1165,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @throws ClassNotFoundException Serialization error
      */
     protected void handleSESSION_DELTA(SessionMessage msg, Member sender) throws IOException, ClassNotFoundException {
-        counterReceive_EVT_SESSION_DELTA++;
+        if (enableStatistics) {
+            counterReceive_EVT_SESSION_DELTA.incrementAndGet();
+        }
         byte[] delta = msg.getSession();
         DeltaSession session = (DeltaSession) findSession(msg.getSessionID());
         if (session == null) {
@@ -1168,7 +1192,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @throws IOException Propagated IO error
      */
     protected void handleSESSION_ACCESSED(SessionMessage msg, Member sender) throws IOException {
-        counterReceive_EVT_SESSION_ACCESSED++;
+        if (enableStatistics) {
+            counterReceive_EVT_SESSION_ACCESSED.incrementAndGet();
+        }
         DeltaSession session = (DeltaSession) findSession(msg.getSessionID());
         if (session != null) {
             if (log.isDebugEnabled()) {
@@ -1181,7 +1207,7 @@ public class DeltaManager extends ClusterManagerBase {
     }
 
     /**
-     * handle receive session is expire at other node ( expire session also here)
+     * handle receive session is expired at other node ( expire session also here)
      *
      * @param msg    Session message
      * @param sender Member which sent the message
@@ -1189,7 +1215,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @throws IOException Propagated IO error
      */
     protected void handleSESSION_EXPIRED(SessionMessage msg, Member sender) throws IOException {
-        counterReceive_EVT_SESSION_EXPIRED++;
+        if (enableStatistics) {
+            counterReceive_EVT_SESSION_EXPIRED.incrementAndGet();
+        }
         DeltaSession session = (DeltaSession) findSession(msg.getSessionID());
         if (session != null) {
             if (log.isDebugEnabled()) {
@@ -1206,7 +1234,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @param sender Member which sent the message
      */
     protected void handleSESSION_CREATED(SessionMessage msg, Member sender) {
-        counterReceive_EVT_SESSION_CREATED++;
+        if (enableStatistics) {
+            counterReceive_EVT_SESSION_CREATED.incrementAndGet();
+        }
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.receiveMessage.createNewSession", getName(), msg.getSessionID()));
         }
@@ -1234,7 +1264,9 @@ public class DeltaManager extends ClusterManagerBase {
      */
     protected void handleALL_SESSION_DATA(SessionMessage msg, Member sender)
             throws ClassNotFoundException, IOException {
-        counterReceive_EVT_ALL_SESSION_DATA++;
+        if (enableStatistics) {
+            counterReceive_EVT_ALL_SESSION_DATA.incrementAndGet();
+        }
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.receiveMessage.allSessionDataBegin", getName()));
         }
@@ -1256,7 +1288,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @throws IOException IO error sending messages
      */
     protected void handleGET_ALL_SESSIONS(SessionMessage msg, Member sender) throws IOException {
-        counterReceive_EVT_GET_ALL_SESSIONS++;
+        if (enableStatistics) {
+            counterReceive_EVT_GET_ALL_SESSIONS.incrementAndGet();
+        }
         // get a list of all the session from this manager
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.receiveMessage.unloadingBegin", getName()));
@@ -1281,6 +1315,7 @@ public class DeltaManager extends ClusterManagerBase {
                     try {
                         Thread.sleep(getSendAllSessionsWaitTime());
                     } catch (Exception sleep) {
+                        // Ignore
                     }
                 }
             }
@@ -1292,7 +1327,9 @@ public class DeltaManager extends ClusterManagerBase {
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.createMessage.allSessionTransferred", getName()));
         }
-        counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE++;
+        if (enableStatistics) {
+            counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE.incrementAndGet();
+        }
         cluster.send(newmsg, sender);
     }
 
@@ -1305,7 +1342,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @throws IOException IO error with serialization
      */
     protected void handleCHANGE_SESSION_ID(SessionMessage msg, Member sender) throws IOException {
-        counterReceive_EVT_CHANGE_SESSION_ID++;
+        if (enableStatistics) {
+            counterReceive_EVT_CHANGE_SESSION_ID.incrementAndGet();
+        }
         DeltaSession session = (DeltaSession) findSession(msg.getSessionID());
         if (session != null) {
             String newSessionID = deserializeSessionId(msg.getSession());
@@ -1323,7 +1362,9 @@ public class DeltaManager extends ClusterManagerBase {
      * @param sender Member which sent the message
      */
     protected void handleALL_SESSION_NOCONTEXTMANAGER(SessionMessage msg, Member sender) {
-        counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER++;
+        if (enableStatistics) {
+            counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER.incrementAndGet();
+        }
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.receiveMessage.noContextManager", getName(), sender.getHost(),
                     Integer.valueOf(sender.getPort())));
@@ -1351,7 +1392,9 @@ public class DeltaManager extends ClusterManagerBase {
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("deltaManager.createMessage.allSessionData", getName()));
         }
-        counterSend_EVT_ALL_SESSION_DATA++;
+        if (enableStatistics) {
+            counterSend_EVT_ALL_SESSION_DATA.incrementAndGet();
+        }
         int sendOptions = Channel.SEND_OPTIONS_SYNCHRONIZED_ACK | Channel.SEND_OPTIONS_USE_ACK;
         cluster.send(newmsg, sender, sendOptions);
     }
