@@ -33,11 +33,11 @@ import java.util.Locale;
 import java.util.Set;
 import java.util.function.Function;
 
-import javax.servlet.ServletException;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
-import javax.servlet.http.HttpSession;
-import javax.servlet.http.Part;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
+import jakarta.servlet.http.HttpSession;
+import jakarta.servlet.http.Part;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
@@ -62,6 +62,10 @@ import org.apache.tomcat.util.security.Escape;
  * However if you use a software that parses the output of <code>ManagerServlet</code> you won't be able to upgrade to
  * this Servlet since the output are not in the same format ar from <code>ManagerServlet</code>
  *
+ * @author Bip Thelin
+ * @author Malcolm Edgar
+ * @author Glenn L. Nielsen
+ *
  * @see ManagerServlet
  */
 public class HTMLManagerServlet extends ManagerServlet {
@@ -266,7 +270,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param cn       Name of the application to be deployed
      * @param war      URL of the web application archive to be deployed
      * @param smClient internationalized strings
-     *
      * @return message String
      */
     protected String deployInternal(String config, ContextName cn, String war, StringManager smClient) {
@@ -287,7 +290,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param response The response
      * @param message  a message to display
      * @param smClient internationalized strings
-     *
      * @throws IOException an IO error occurred
      */
     protected void list(HttpServletRequest request, HttpServletResponse response, String message,
@@ -566,9 +568,7 @@ public class HTMLManagerServlet extends ManagerServlet {
      *
      * @param cn       Name of the application to be restarted
      * @param smClient StringManager for the client's locale
-     *
      * @return message String
-     *
      * @see ManagerServlet#reload(PrintWriter, ContextName, StringManager)
      */
     protected String reload(ContextName cn, StringManager smClient) {
@@ -586,9 +586,7 @@ public class HTMLManagerServlet extends ManagerServlet {
      *
      * @param cn       Name of the application to be undeployed
      * @param smClient StringManager for the client's locale
-     *
      * @return message String
-     *
      * @see ManagerServlet#undeploy(PrintWriter, ContextName, StringManager)
      */
     protected String undeploy(ContextName cn, StringManager smClient) {
@@ -607,9 +605,7 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param cn       Name of the application to list session information
      * @param idle     Expire all sessions with idle time &ge; idle for this context
      * @param smClient StringManager for the client's locale
-     *
      * @return message String
-     *
      * @see ManagerServlet#sessions(PrintWriter, ContextName, int, StringManager)
      */
     protected String sessions(ContextName cn, int idle, StringManager smClient) {
@@ -627,9 +623,7 @@ public class HTMLManagerServlet extends ManagerServlet {
      *
      * @param cn       Name of the application to be started
      * @param smClient StringManager for the client's locale
-     *
      * @return message String
-     *
      * @see ManagerServlet#start(PrintWriter, ContextName, StringManager)
      */
     protected String start(ContextName cn, StringManager smClient) {
@@ -647,9 +641,7 @@ public class HTMLManagerServlet extends ManagerServlet {
      *
      * @param cn       Name of the application to be stopped
      * @param smClient StringManager for the client's locale
-     *
      * @return message String
-     *
      * @see ManagerServlet#stop(PrintWriter, ContextName, StringManager)
      */
     protected String stop(ContextName cn, StringManager smClient) {
@@ -666,9 +658,7 @@ public class HTMLManagerServlet extends ManagerServlet {
      * Find potential memory leaks caused by web application reload.
      *
      * @param smClient StringManager for the client's locale
-     *
      * @return message String
-     *
      * @see ManagerServlet#findleaks(boolean, PrintWriter, StringManager)
      */
     protected String findleaks(StringManager smClient) {
@@ -728,7 +718,7 @@ public class HTMLManagerServlet extends ManagerServlet {
 
     @Override
     public String getServletInfo() {
-        return "HTMLManagerServlet, Copyright (c) 1999-2026, The Apache Software Foundation";
+        return "HTMLManagerServlet, Copyright (c) 1999-2025, The Apache Software Foundation";
     }
 
     @Override
@@ -750,7 +740,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param cn       Name of the application from which to expire sessions
      * @param req      The Servlet request
      * @param smClient StringManager for the client's locale
-     *
      * @return message string
      */
     protected String expireSessions(ContextName cn, HttpServletRequest req, StringManager smClient) {
@@ -773,7 +762,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param req      The Servlet request
      * @param resp     The Servlet response
      * @param smClient StringManager for the client's locale
-     *
      * @throws ServletException Propagated Servlet error
      * @throws IOException      An IO error occurred
      */
@@ -859,7 +847,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param req      The Servlet request
      * @param resp     The Servlet response
      * @param smClient StringManager for the client's locale
-     *
      * @throws ServletException Propagated Servlet error
      * @throws IOException      An IO error occurred
      */
@@ -909,7 +896,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param cn        Name of the application for which the sessions will be listed
      * @param sessionId the session id
      * @param smClient  StringManager for the client's locale
-     *
      * @throws ServletException Propagated Servlet error
      * @throws IOException      An IO error occurred
      */
@@ -932,7 +918,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param cn         Name of the application for which sessions are to be invalidated
      * @param sessionIds the session ids of the sessions
      * @param smClient   StringManager for the client's locale
-     *
      * @return number of invalidated sessions
      */
     protected int invalidateSessions(ContextName cn, String[] sessionIds, StringManager smClient) {
@@ -971,7 +956,6 @@ public class HTMLManagerServlet extends ManagerServlet {
      * @param sessionId     the session id
      * @param attributeName the attribute name
      * @param smClient      StringManager for the client's locale
-     *
      * @return true if there was an attribute removed, false otherwise
      */
     protected boolean removeSessionAttribute(ContextName cn, String sessionId, String attributeName,
