@@ -27,9 +27,6 @@ import java.security.cert.CertificateException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -89,7 +86,7 @@ public class AjpProcessor extends AbstractProcessor {
     private static final byte[] pongMessageArray;
 
 
-    private static final Map<String, String> jakartaAttributeMapping;
+    private static final Map<String,String> jakartaAttributeMapping;
     private static final Set<String> iisTlsAttributes;
 
 
@@ -131,28 +128,17 @@ public class AjpProcessor extends AbstractProcessor {
         System.arraycopy(pongMessage.getBuffer(), 0, pongMessageArray, 0, pongMessage.getLen());
 
         // Build Map of Java Servlet to Jakarta Servlet attribute names
-        Map<String, String> m = new HashMap<>();
-        m.put("jakarta.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite");
-        m.put("jakarta.servlet.request.key_size", "jakarta.servlet.request.key_size");
-        m.put("jakarta.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session");
-        m.put("jakarta.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
-        m.put("javax.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite");
-        m.put("javax.servlet.request.key_size", "jakarta.servlet.request.key_size");
-        m.put("javax.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session");
-        m.put("javax.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
-        jakartaAttributeMapping = Collections.unmodifiableMap(m);
+        jakartaAttributeMapping = Map.of("jakarta.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite",
+                "jakarta.servlet.request.key_size", "jakarta.servlet.request.key_size",
+                "jakarta.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session",
+                "jakarta.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate",
+                "javax.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite",
+                "javax.servlet.request.key_size", "jakarta.servlet.request.key_size",
+                "javax.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session",
+                "javax.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
 
-        Set<String> s = new HashSet<>();
-        s.add("CERT_ISSUER");
-        s.add("CERT_SUBJECT");
-        s.add("CERT_COOKIE");
-        s.add("HTTPS_SERVER_SUBJECT");
-        s.add("CERT_FLAGS");
-        s.add("HTTPS_SECRETKEYSIZE");
-        s.add("CERT_SERIALNUMBER");
-        s.add("HTTPS_SERVER_ISSUER");
-        s.add("HTTPS_KEYSIZE");
-        iisTlsAttributes = Collections.unmodifiableSet(s);
+        iisTlsAttributes = Set.of("CERT_ISSUER", "CERT_SUBJECT", "CERT_COOKIE", "HTTPS_SERVER_SUBJECT", "CERT_FLAGS",
+                "HTTPS_SECRETKEYSIZE", "CERT_SERIALNUMBER", "HTTPS_SERVER_ISSUER", "HTTPS_KEYSIZE");
     }
 
 
@@ -678,7 +664,7 @@ public class AjpProcessor extends AbstractProcessor {
         boolean contentLengthSet = false;
         int hCount = requestHeaderMessage.getInt();
         for (int i = 0; i < hCount; i++) {
-            String hName = null;
+            String hName;
 
             // Header names are encoded as either an integer code starting
             // with 0xA0, or as a normal string (in which case the first
@@ -686,7 +672,7 @@ public class AjpProcessor extends AbstractProcessor {
             int isc = requestHeaderMessage.peekInt();
             int hId = isc & 0xFF;
 
-            MessageBytes vMB = null;
+            MessageBytes vMB;
             isc &= 0xFF00;
             if (0xA000 == isc) {
                 requestHeaderMessage.getInt(); // To advance the read position
@@ -720,7 +706,7 @@ public class AjpProcessor extends AbstractProcessor {
             } else if (hId == Constants.SC_REQ_CONTENT_TYPE || (hId == -1 && tmpMB.equalsIgnoreCase("Content-Type"))) {
                 // just read the content-type header, so set it
                 ByteChunk bchunk = vMB.getByteChunk();
-                request.contentType().setBytes(bchunk.getBytes(), bchunk.getOffset(), bchunk.getLength());
+                request.contentType().setBytes(bchunk.getBytes(), bchunk.getStart(), bchunk.getLength());
             }
         }
 
@@ -841,7 +827,7 @@ public class AjpProcessor extends AbstractProcessor {
 
                 case Constants.SC_A_SECRET:
                     requestHeaderMessage.getBytes(tmpMB);
-                    if (secret != null && secret.length() > 0) {
+                    if (secret != null && !secret.isEmpty()) {
                         secretPresentInRequest = true;
                         if (!tmpMB.equals(secret)) {
                             response.setStatus(403);
@@ -859,7 +845,7 @@ public class AjpProcessor extends AbstractProcessor {
         }
 
         // Check if secret was submitted if required
-        if (secret != null && secret.length() > 0 && !secretPresentInRequest) {
+        if (secret != null && !secret.isEmpty() && !secretPresentInRequest) {
             response.setStatus(403);
             setErrorState(ErrorState.CLOSE_CLEAN, null);
         }
@@ -870,10 +856,9 @@ public class AjpProcessor extends AbstractProcessor {
 
             int pos = uriBC.indexOf("://", 0, 3, 4);
             int uriBCStart = uriBC.getStart();
-            int slashPos = -1;
             if (pos != -1) {
                 byte[] uriB = uriBC.getBytes();
-                slashPos = uriBC.indexOf('/', pos + 3);
+                int slashPos = uriBC.indexOf('/', pos + 3);
                 if (slashPos == -1) {
                     slashPos = uriBC.getLength();
                     // Set URI as "/"
@@ -991,8 +976,8 @@ public class AjpProcessor extends AbstractProcessor {
                     responseMessage.appendBytes(hV);
                 } catch (IllegalArgumentException iae) {
                     // Log the problematic header
-                    log.warn(sm.getString("ajpprocessor.response.invalidHeader", headers.getName(i), headers.getValue(i)),
-                            iae);
+                    log.warn(sm.getString("ajpprocessor.response.invalidHeader", headers.getName(i),
+                            headers.getValue(i)), iae);
                     // Remove the problematic header
                     headers.removeHeader(i);
                     numHeaders--;
@@ -1010,9 +995,6 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
-    /**
-     * Callback to write data from the buffer.
-     */
     @Override
     protected final void flush() throws IOException {
         // Calling code should ensure that there is no data in the buffers for
@@ -1028,9 +1010,6 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
-    /**
-     * Finish AJP response.
-     */
     @Override
     protected final void finishResponse() throws IOException {
         if (responseFinished) {
@@ -1060,6 +1039,12 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
+    @Override
+    protected void earlyHints() throws IOException {
+        // NO-OP for AJP
+    }
+
+
     @Override
     protected final int available(boolean doRead) {
         if (endOfStream) {
