@@ -35,6 +35,7 @@ import org.apache.coyote.ActionCode;
 import org.apache.coyote.Adapter;
 import org.apache.coyote.ContinueResponseTiming;
 import org.apache.coyote.ErrorState;
+import org.apache.coyote.NonPipeliningProcessor;
 import org.apache.coyote.Request;
 import org.apache.coyote.RequestGroupInfo;
 import org.apache.coyote.Response;
@@ -52,7 +53,7 @@ import org.apache.tomcat.util.net.SocketEvent;
 import org.apache.tomcat.util.net.SocketWrapperBase;
 import org.apache.tomcat.util.res.StringManager;
 
-class StreamProcessor extends AbstractProcessor {
+class StreamProcessor extends AbstractProcessor implements NonPipeliningProcessor {
 
     private static final Log log = LogFactory.getLog(StreamProcessor.class);
     private static final StringManager sm = StringManager.getManager(StreamProcessor.class);
@@ -146,8 +147,14 @@ class StreamProcessor extends AbstractProcessor {
                     state = SocketState.CLOSED;
                 } finally {
                     if (state == SocketState.CLOSED) {
-                        stream.recycle();
+                        /*
+                         * Recycle this processor before the stream is recycled as recycling the stream will add the
+                         * request and the response to the pool for re-use (if re-use is enabled) and the request
+                         * statistics updating in StreamProcessor.recycle() needs to happen before the request and
+                         * response are added to the pool to avoid concurrency issues corrupting the statistics.
+                         */
                         recycle();
+                        stream.recycle();
                     }
                 }
             } finally {
@@ -255,12 +262,6 @@ class StreamProcessor extends AbstractProcessor {
                 headers.setValue("Server").setString(server);
             }
         }
-
-        // Remove payload headers for HEAD requests
-        if (coyoteRequest != null && "HEAD".equals(coyoteRequest.method().toString())) {
-            headers.removeHeader("content-length");
-            headers.removeHeader("content-range");
-        }
     }
 
 
@@ -438,6 +439,12 @@ class StreamProcessor extends AbstractProcessor {
             global.removeRequestProcessor(request.getRequestProcessor());
         }
 
+        /*
+         * Clear the statistics ready for re-use of the request. If we don't clear the statistics, the statistics for
+         * the current request will be included in the statistics for all future requests.
+         */
+        request.getRequestProcessor().recycleStatistcs();
+
         // Clear fields that can be cleared to aid GC and trigger NPEs if this
         // is reused
         setSocketWrapper(null);
