@@ -27,8 +27,8 @@ import java.net.URLConnection;
 import java.util.Set;
 import java.util.jar.JarEntry;
 
-import javax.servlet.ServletContext;
-import javax.servlet.jsp.tagext.TagInfo;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.jsp.tagext.TagInfo;
 
 import org.apache.jasper.compiler.Compiler;
 import org.apache.jasper.compiler.JspRuntimeContext;
@@ -124,7 +124,7 @@ public class JspCompilationContext {
         this.baseURI = baseURI;
 
         this.rctxt = rctxt;
-        this.basePackageName = Constants.JSP_PACKAGE_NAME;
+        this.basePackageName = options.getGeneratedJspPackageName();
 
         this.tagInfo = tagInfo;
         this.tagJar = tagJar;
@@ -173,7 +173,8 @@ public class JspCompilationContext {
 
     public ClassLoader getJspLoader() {
         if (jspLoader == null) {
-            jspLoader = new JasperLoader(new URL[] { baseUrl }, getClassLoader(), rctxt.getPermissionCollection());
+            jspLoader = new JasperLoader(new URL[] { baseUrl }, getClassLoader(), basePackageName,
+                    rctxt.getPermissionCollection());
         }
         return jspLoader;
     }
@@ -391,17 +392,17 @@ public class JspCompilationContext {
                     result = uc.getLastModified();
                 }
             }
-        } catch (IOException ioe) {
+        } catch (IOException e) {
             if (log.isDebugEnabled()) {
-                log.debug(Localizer.getMessage("jsp.error.lastModified", getJspFile()), ioe);
+                log.debug(Localizer.getMessage("jsp.error.lastModified", getJspFile()), e);
             }
         } finally {
             if (uc != null) {
                 try {
                     uc.getInputStream().close();
-                } catch (IOException ioe) {
+                } catch (IOException e) {
                     if (log.isDebugEnabled()) {
-                        log.debug(Localizer.getMessage("jsp.error.lastModified", getJspFile()), ioe);
+                        log.debug(Localizer.getMessage("jsp.error.lastModified", getJspFile()), e);
                     }
                     result = -1;
                 }
@@ -601,8 +602,8 @@ public class JspCompilationContext {
             } catch (FileNotFoundException fnfe) {
                 // Re-throw to let caller handle this - will result in a 404
                 throw fnfe;
-            } catch (Exception e) {
-                JasperException je = new JasperException(Localizer.getMessage("jsp.error.unable.compile"), e);
+            } catch (Exception ex) {
+                JasperException je = new JasperException(Localizer.getMessage("jsp.error.unable.compile"), ex);
                 // Cache compilation exception
                 jsw.setCompilationException(je);
                 throw je;
@@ -622,8 +623,8 @@ public class JspCompilationContext {
             servletClass = jspLoader.loadClass(name);
         } catch (ClassNotFoundException cex) {
             throw new JasperException(Localizer.getMessage("jsp.error.unable.load"), cex);
-        } catch (Exception e) {
-            throw new JasperException(Localizer.getMessage("jsp.error.unable.compile"), e);
+        } catch (Exception ex) {
+            throw new JasperException(Localizer.getMessage("jsp.error.unable.compile"), ex);
         }
         removed = false;
         return servletClass;
