@@ -25,7 +25,7 @@ import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.apache.tomcat.util.http.parser.TokenList;
 
@@ -82,8 +82,8 @@ public class ResponseUtil {
             StringReader input = new StringReader(varyHeader);
             try {
                 TokenList.parseTokenList(input, fieldNames);
-            } catch (IOException ignore) {
-                // Should never happen because a StringReader is used.
+            } catch (IOException ioe) {
+                // Should never happen
             }
         }
 
